"""
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, 
or (at your option) any later version.


This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.


You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.

Copyright © 2019 Cloud Linux Software Inc.

This software is also available under ImunifyAV commercial license,
see <https://www.imunify360.com/legal/eula>
"""
import time
from collections import Counter
from pathlib import Path
from typing import List

from defence360agent.contracts.config import (
    IContactMessageType,
    MalwareScanSchedule
)
from defence360agent.contracts.hook_events import HookEvent
from defence360agent.contracts.messages import IContactSent
from defence360agent.contracts.plugins import (
    MessageSink,
    MessageSource,
    expect,
)
from defence360agent.internals.the_sink import TheSink
from defence360agent.malwarelib.config import MalwareScanType
from defence360agent.model.icontact import IContactThrottle
from defence360agent.subsys.panels.cpanel import cPanel
from defence360agent.subsys.panels.hosting_panel import HostingPanel
from defence360agent.utils import (
    create_task_and_log_exceptions,
    recurring_check,
)
from defence360agent.utils.common import DAY


class IContactSender(MessageSink, MessageSource):
    PROCESSING_ORDER = MessageSink.ProcessingOrder.ICONTACT_SENT

    async def create_sink(self, loop):
        pass

    async def _send_icontact_message(self, *, message_type, params):
        if message_type is None:
            return
        if not IContactThrottle.may_be_notified(message_type):
            return
        template_args = await self._panel.notify(
            message_type=message_type, params=params
        )
        if template_args:
            IContactThrottle.refresh(message_type)
            sent_message = IContactSent(
                message_type=message_type,
                timestamp=int(time.time()),
                template_args=template_args,
            )
            await self._sink.process_message(sent_message)

    async def create_source(self, loop, sink: TheSink):
        self._sink = sink
        self._panel = HostingPanel()
        if self._panel.NAME == cPanel.NAME:
            self._task = create_task_and_log_exceptions(
                loop, self.no_scan_schedule
            )
        else:
            self._task = None

    async def shutdown(self):
        if self._task:
            self._task.cancel()
            await self._task

    @expect(HookEvent.MalwareDetectedCritical)
    async def malware_found(self, event):
        if event["scan_type"] != MalwareScanType.BACKGROUND:
            return

        if self._panel.NAME != cPanel.NAME:
            return

        hits: List[dict] = event.get("DUMP")
        if not hits:
            return

        counter = Counter(hit["username"] for hit in hits)
        infected_data = counter.most_common(1)
        infected_user = infected_data[0][0]

        await self._send_icontact_message(
            message_type=IContactMessageType.MALWARE_FOUND,
            params={"infected_user": infected_user}
        )

    @recurring_check(DAY)
    async def no_scan_schedule(self):
        try:
            cron_contains_scheduled_scan = (
                MalwareScanSchedule.CMD in
                Path(MalwareScanSchedule.CRON_PATH).read_text()
            )
        except FileNotFoundError:
            cron_contains_scheduled_scan = False

        if not cron_contains_scheduled_scan:
            await self._send_icontact_message(
                message_type=IContactMessageType.SCAN_NOT_SCHEDULED,
                params={}
            )
